function hout = fibrplot(fibr,t,ix,s)
%FIBRPLOT Plot fibers in Stokes flow.
%   H = FIBRPLOT(FIBR,T) plots the fibers at time T and returns a
%   handle to the resulting plot.
%
%   FIBRPLOT(FIBR,T,...) sends any additional arguments to PLOT3.
%
%   FIBRPLOT(FIBR,T,IX,...) only plots the fibers with index IX.
%
%   See also PLOT3, ADVECT, FIBRSETUP.

% S. Engblom 2010-01-15 (Index and plot3-argument added)
% S. Engblom 2009-06-08
% A-K. Tornberg (original code 'plot_data')

% additional arguments
if nargin < 4
  s = '';
  if nargin < 3
    ix = 1:fibr.Mfibers;
  elseif ischar(ix)
    s = ix;
    ix = 1:fibr.Mfibers;
  end
end

% find endpoints
[X,T] = compute(fibr,t,'r','t',ix);
xx = tsum(X,cat(3,-T,T),[3 2],[3 2 1]);

% plot
h = plot3(xx(:,:,1),xx(:,:,2),xx(:,:,3),s);
set(h,'LineWidth',2);
axis equal;
axis(reshape([-fibr.Ldim/2; fibr.Ldim/2],1,[]));
if nargout > 0, hout = h; end
