function iG = greendotleg2(x,y,z,xc,tv,G,p,wL)
%GREENDOTLEG2 Green's function remainder times Legendre basis.
%   IG = GREENDOTLEG2(X,Y,Z,XC,TV,G,P,wL) computes the integral inner
%   product
%
%     IG = int G([X Y Z]-s) L(s) ds,
%
%   where the integral is over a fiber with center coordinates XC and
%   tangent vector TV. The format of the Green's function remainder G
%   is described in GREENTABLE, the quadrature (P, W) in LEGQUAD and
%   the Legendre polynomials L in LEGEVAL (wL are the polynomials L
%   scaled with w).
%
%   The output IG is a numel(X)-by-(size(L,2)+1)-by-6 3-D array.
%
%   Performance of this function is important. No error-checking is
%   performed.
%
%   See also GREENDOTLEG1, GREENTABLE, LEGQUAD, LEGEVAL.

% S. Engblom 2009-05-11 (Revision)
% A-K. Tornberg (original code 'int_per_vec')

% eval in quadrature points
x = tsum(x(:),-xc(1)-tv(1)*p,2,1);
y = tsum(y(:),-xc(2)-tv(2)*p,2,1);
z = tsum(z(:),-xc(3)-tv(3)*p,2,1);

% interpolate Green's function remainder
iG = greeninterp(x,y,z,G);

% intergrate with Legendre polynomials
iG = tprod(iG,wL,[-1 1 3],[-1 2]);
