function J = jacobian(fibr,t)
%JACOBIAN Compute Jacobian.
%   J = JACOBIAN(FIBR,T) computes the Jacobian at time T. On return, J
%   is a 6*M-by-6*M-by-numel(T) array with J(:,:,i) the Jacobian at
%   time T(i). Here M = fibr.Mfibers is the number of fibers and FIBR
%   must contain a valid solution for the interval T; see ADVECT.
%
%   The purpose of this function is mainly experimental and it should
%   not be used for implicit ODE-solvers. 
%
%   See also FIBRSETUP, ADVECT, COMPUTE, VELOCITY, NUMJAC.

% S. Engblom 2010-02-03

if fibr.report
  report([1 numel(t)*fibr.Mfibers*6],'Computing Jacobian...','init');
else
  report([],[],'none');
end

% right-hand side function already computed, use it
[R,T,dR,dT] = compute(fibr,t,'r','t','dr','dt');

J = zeros(6*fibr.Mfibers,6*fibr.Mfibers,numel(t));
fac = [];
for i = 1:numel(t)
  yi = reshape([R(:,:,i); T(:,:,i)],[],1);
  ftyi = reshape([dR(:,:,i); dT(:,:,i)],[],1);
  [J(:,:,i),fac] = numjac(@l_rhsfun,t(i),yi,ftyi,0,fac,1,[],[],fibr, ...
                          (i-1)*6*fibr.Mfibers);
end
report([],[],'done');

%-------------------------------------------------------------------------
function dy = l_rhsfun(t,y,fibr,offset)
%L_RHSFUN Right-hand side function placeholder.
%  DY = L_RHSFUN(T,Y,FIBR,OFFSET) computes velocities DY at times T in
%  a vectorized fashion. OFFSET is used to control the reporter.

dy = zeros(size(y));
for i = 1:size(y,2)
  yy = reshape(y(:,i),6,[]);
  [dr,dt] = velocity(fibr,yy(1:3,:),yy(4:6,:));
  dy(:,i) = reshape([dr; dt],[],1);
  report(offset+i,[],'');
end

%-------------------------------------------------------------------------
