function [x,w] = legquad(N,Nrep)
%LEGQUAD Gauss-Legendre quadrature.
%   [P,W] = LEGQUAD(N,NREP) returns points P and weights W for a Gauss
%   quadrature with N points in each of NREP intervals on (-1,1).
%
%   Both P and W are column vectors of length N*NREP.

% S. Engblom 2009-07-10 (Revision, arbitrary order)
% S. Engblom 2009-05-11 (Revision)
% A-K. Tornberg (original code 'quad_pts_and_wts')

% construct a symmetric tridiagonal matrix J
nn = 1:N;
b = nn./sqrt(4*nn.*nn-1);
b(end) = 0;
J = fsparse([[1 1:N-1]; [2:N N]],nn, ...
            [b([end 1:end-1]); b],[],'nosort');

% eigenvalues and -vectors
[y,x] = eig(full(J));

% the abscissas are the eigenvalues of J
x = diag(x);

% the weights can be found from the first element of each eigenvector
w = 2*reshape(y(1,:).^2,[],1);

% repeat this
if nargin > 1 && Nrep > 1
  xx = linspace(-1,1,Nrep+1)';
  xx(end) = [];
  xx = xx+0.5*(xx(2)-xx(1));
  x = reshape(tsum(xx,x/Nrep,2,1),[],1);
  w = frepmat(w/Nrep,Nrep);
end
