function spin(ix)
%SPIN Tests for FIBR.

% S. Engblom 2009-05-12

ftests = {@l_spin1 @l_spin2 @l_spin3 @l_spin4};
n = size(ftests,2);
stests = cellstr(reshape(sprintf('Test #%2d',1:n)',[],n)')';
if nargin == 0, ix = 1:n; end
runtest('SPIN (fibr)',ftests(ix),stests(ix));

%--------------------------------------------------------------------------
function ok = l_spin1
%L_SPIN1 Test generation of Green's function table.

ok = 1;

Ldim = [4 4 4];
h = 0.5;
G = greentable(Ldim,Ldim/2+1,[2 h]);

% compare to saved values
load spin1

ok = ok && all(abs(G.G(:)-Gsave1(:)) <= ...
               max(0.5e-8*abs(Gsave1(:)),eps(1000)));

% less symmetric case
Ldim = [3 5.75 4.25];
h = 0.5;
G = greentable(Ldim,Ldim/2+1,[2 h]);

ok = ok && all(abs(G.G(:)-Gsave2(:)) <= ...
               max(0.5e-8*abs(Gsave2(:)),eps(1000)));

%--------------------------------------------------------------------------
function ok = l_spin2
%L_SPIN2 Test generation of Green's function table (Chebyshev case).

ok = 1;

return; % *** unfinished

Ldim = [4 4 4];
G = greentable(Ldim,Ldim/2+1,[],6);

% saved G
load spin1
Gsave.G = Gsave1;
Gsave.htable = 0.5;
Gsave.sizG = [7 7 7];

% midpoints where linear interpolation does about worst
xx = (0:0.25:3)'; xx(1:2:end) = [];
Gx = greenstokes(xx,xx,xx,Ldim,'report',1);
[X,Y,Z] = ndgrid(xx);
Gg = greeninterp(X(:),Y(:),Z(:),G);
Gg = reshape(Gg,size(Gx));
Gs = greeninterp(X(:),Y(:),Z(:),Gsave);
Gs = reshape(Gs,size(Gx));

% check
norm(Gg(:)-Gx(:),inf)
norm(Gs(:)-Gx(:),inf)

% plot
x = linspace(0,3,80)'; x(end) = [];
y = 2; z = 2;
yy = frepmat(y,size(x)); zz = frepmat(z,size(x));
g1 = reshape(greeninterp(x,y,z,G),[],6);
g2 = reshape(greeninterp(x,yy,zz,Gsave),[],6);
gx = reshape(greenstokes(x,y,z,Ldim),[],6);
plot(x,g1-gx,'r',x,g2-gx,'b')

% *** more accurate case

Ldim = [4 4 4];
G = greentable(Ldim,Ldim/2+1,[],10,'report',true);
G4 = greentable(Ldim,Ldim/2+1,0.2,4,'report',true);

% saved G
load scrub1
Gsave.G = Gsave;
Gsave.htable = 0.2;
Gsave.Ntable = 2;
Gsave.sizG = [16 16 16];

% midpoints where linear interpolation does about worst
xx = (0:0.1:3)'; xx(1:2:end) = [];
Gx = greenstokes(xx,xx,xx,Ldim,'report',1);
[X,Y,Z] = ndgrid(xx);
Gg = greeninterp(X(:),Y(:),Z(:),G);
Gg = reshape(Gg,size(Gx));
Gg4 = greeninterp(X(:),Y(:),Z(:),G4);
Gg4 = reshape(Gg4,size(Gx));
Gs = greeninterp(X(:),Y(:),Z(:),Gsave);
Gs = reshape(Gs,size(Gx));

% check
norm(Gg(:)-Gx(:),1)
norm(Gg4(:)-Gx(:),1)
norm(Gs(:)-Gx(:),1)

% plot
x = linspace(0,3,80)'; x(end) = [];
y = 2; z = 2;
yy = frepmat(y,size(x)); zz = frepmat(z,size(x));
g1 = reshape(greeninterp(x,y,z,G),[],6);
g4 = reshape(greeninterp(x,yy,zz,G4),[],6);
g2 = reshape(greeninterp(x,yy,zz,Gsave),[],6);
gx = reshape(greenstokes(x,y,z,Ldim),[],6);
c = {'b' 'r' 'g' 'k' 'm' 'c'};
d = {'b--' 'r--' 'g--' 'k--' 'm--' 'c--'};
e = {'b:' 'r:' 'g:' 'k:' 'm:' 'c:'};
clf
for i = 1:6
  semilogy(x,abs(g1(:,i)-gx(:,i)),c{i}, ...
           x,abs(g4(:,i)-gx(:,i)),d{i}, ...
           x,abs(g2(:,i)-gx(:,i)),e{i}), hold on
end
hold off

ok = ok && all(abs(G.G(:)-Gsave1(:)) <= ...
               max(0.5e-8*abs(Gsave1(:)),eps(1000)));

%--------------------------------------------------------------------------
function ok = l_spin3
%L_SPIN3 Small check, 5 fibers, 5 time-steps.

ok = 1;
rand('state',2008);

% used saved table
load spin1
fibr.Ldim = [4 4 4];
fibr.ntable = [2 0.5];
Lmax = fibr.Ldim/2+1; % works here, but is not the default...
fibr.Green.G = Gsave1;
fibr.Green.sizG = ceil(Lmax/fibr.ntable(2))+1;
fibr.Mfibers = 5;
fibr.report = false;

% setup and solve
fibr = fibrsetup(fibr);
fibr.init = randfibr(fibr);
fibr.sol = advect(fibr,0:0.1:0.5);

% precomputed solution
load spin3
err = abs(fibr.sol.sol-fibrdof);

% criterion (can be questioned!)
ok = ok && all(err(:) <= 0.5e-6*abs(fibrdof(:)));

%--------------------------------------------------------------------------
function ok = l_spin4
%L_SPIN4 DISTANCE.

ok = 1;

% test second syntax through first syntax
theta = linspace(0,2*pi,7);
X2 = [cos(theta); sin(theta); zeros(size(theta))];
X2 = [X2 X2 X2 X2];
T2 = [zeros(2,size(theta,2)); ones(size(theta))];
T2 = [T2 -T2 T2 -T2];
XT2 = [X2; T2];

D1 = distances(XT2,2);
D2 = distances(XT2,XT2,2);
D = D1-tril(D2,-1);
ok = ok && norm(D,inf) < 1e-12;

X2(3,:) = X2(3,:)+1;
XT2 = [X2; T2];
D1 = distances(XT2,2);
D2 = distances(XT2,XT2,2);
D = D1-tril(D2,-1);
ok = ok && norm(D,inf) < 1e-12;

X2(3,:) = X2(3,:)+2;
XT2 = [X2; T2];
D1 = distances(XT2,2);
D2 = distances(XT2,XT2,2);
D = D1-tril(D2,-1);
ok = ok && norm(D,inf) < 1e-12;

T2 = [cos(theta); sin(theta); ones(size(theta))]/sqrt(2);
T2 = [T2 -T2 T2 -T2];
XT2 = [X2; T2];
D1 = distances(XT2,2);
D2 = distances(XT2,XT2,2);
D = D1-tril(D2,-1);
ok = ok && norm(D,inf) < 1e-12;

rand('state',299);
R = rand(3);
[R,D] = eig(R'+R);

XT2 = [R*XT2(1:3,:); R*XT2(4:6,:)];
D1 = distances(XT2,2);
D2 = distances(XT2,XT2,2);
D = D1-tril(D2,-1);
ok = ok && norm(D,inf) < 1e-12;

X2 = [zeros(2,4); 0.23*ones(1,4)];
T2 = [zeros(1,4); 1 -1 1 -1; zeros(1,4)];
XT2 = [X2; T2];
XT2 = [R*XT2(1:3,:); R*XT2(4:6,:)];
D1 = distances(XT2,2);
D2 = distances(XT2,XT2,2);
D = D1-tril(D2,-1);
ok = ok && norm(D,inf) < 1e-12;

X2 = [zeros(1,4); 1.5*ones(2,4)];
T2 = [zeros(1,4); 1 -1 1 -1; zeros(1,4)];
XT2 = [X2; T2];
XT2 = [R*XT2(1:3,:); R*XT2(4:6,:)];
D1 = distances(XT2,2);
D2 = distances(XT2,XT2,2);
D = D1-tril(D2,-1);
ok = ok && norm(D,inf) < 1e-12;

X2 = [ones(1,4); zeros(1,4); 0.53*ones(1,4)];
T2 = [zeros(1,4); 1 -1 1 -1; zeros(1,4)];
XT2 = [X2; T2];
XT2 = [R*XT2(1:3,:); R*XT2(4:6,:)];
D1 = distances(XT2,2);
D2 = distances(XT2,XT2,2);
D = D1-tril(D2,-1);
ok = ok && norm(D,inf) < 1e-12;

%--------------------------------------------------------------------------
