function [V,R] = velocity(fibr,xc,tv)
%VELOCITY Computation of velocities of fibers in Stokes flow.
%   [V,R] = VELOCITY(FIBR,XC,TV) computes the transational (V) and the
%   rotational (R) velocities of the fibers. The arguments XC and TV
%   are explained in ASSEMBLE.
%
%   See also ASSEMBLE, FIBRSETUP.

% S. Engblom 2009-05-11 (Revision)
% A-K. Tornberg (original codes 'compute_velocities_per' and
% 'solve_coeff_per')

% all job is really done by assemble:
t1 = cputime;
[A,b,V,R,CV,CR] = ...
    assemble(xc,tv,fibr.Ldim,fibr.eeps,fibr.ExtForce,fibr.Green, ...
             fibr.Legendre.p,fibr.Legendre.wL,fibr.Legendre.M2L,fibr.aux);
fdof = A\b;
V = reshape(V(:)'+(CV*fdof(:))',3,[]);
R = reshape(R(:)'+(CR*fdof(:))',3,[]);
t2 = cputime;
if fibr.report
  fprintf('Assembling, solving and computing velocities: %fs.\n',t2-t1);
end
